"""subscriber: add bounce column to count bounces

Revision ID: 567ebef5513b
Revises: 000000000000
Create Date: 2025-12-15 16:11:02.092432

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "567ebef5513b"
down_revision = "000000000000"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("subscriber", schema=None) as batch_op:
        batch_op.add_column(sa.Column("bounces", sa.Integer(), server_default="0"))

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("subscriber", schema=None) as batch_op:
        batch_op.drop_column("bounces")

    # ### end Alembic commands ###
