{% extends "base.html" %}

{% block title %}{{ _('Login') }}{% endblock %}

{% block content %}
<h2>{{ _('Login') }}</h2>

{%- include "flash.html" %} {# Show flash messages #}

<form method="post">
  {{ form.csrf_token }}

  {{ form.username.label }}
  {{ form.username() }}

  {{ form.password.label }}
  {{ form.password() }}

  {{ form.submit }}
</form>
{% endblock %}
