{% extends "base.html" %}

{% block title %}{{ _('Edit List Settings') }} - {{ mailing_list.display }}{% endblock %}

{% block content %}
<h2>{{ _('Edit List Settings') }}: {{ mailing_list.display }}</h2>

{%- include "flash.html" %} {# Show flash messages #}

<p>
    <a href="{{ url_for('lists.subscribers_manage', list_id=mailing_list.id) }}" role="button">{{ _('View List Subscribers') }}</a>
</p>

{%- from "macros.html" import list_form %}
{{ list_form(form, false) }}

{%- endblock %}
