# SPDX-FileCopyrightText: 2025 Max Mehl <https://mehl.mx>
#
# SPDX-License-Identifier: Apache-2.0

"""Global constants"""

from importlib.metadata import version

__version__ = version("castmail2list")
