{%- with messages = get_flashed_messages(with_categories=True) %}
{%- if messages %}
<div class="flashes">
  {%- for category, message in messages %}
  <div class="flash {{ category }}">
    {{ message | safe }}
  </div>
  {%- endfor %}
</div>
{%- endif %}
{%- endwith %}
