<!DOCTYPE html>
<html lang="{{ current_language }}">

<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <meta name="color-scheme" content="light dark">
  <title>{% block title %}CastMail2List{% endblock %} - CastMail2List</title>
  <link rel="stylesheet" href="{{ url_for('static', filename='css/fontawesome-all.min.css') }}">
  <link rel="stylesheet" href="{{ url_for('static', filename='css/main.scss.css') }}" />

  <!-- Favicon -->
  <meta name="apple-mobile-web-app-title" content="CastMail2List" />
  {#
  <link rel="icon" type="image/svg+xml" href="/img/favicon.svg" />
  <link rel="shortcut icon" href="/img/favicon.ico" />
  <link rel="apple-touch-icon" sizes="180x180" href="/img/apple-touch-icon.png" />
  <link rel="manifest" href="/img/site.webmanifest" /> #}
  <link rel="icon" type="image/png" href="{{ url_for('static', filename='img/favicon-96x96.png') }}" sizes="96x96" />
  <link rel="shortcut icon" href="{{ url_for('static', filename='img/favicon.ico') }}" />
  <link rel="apple-touch-icon" sizes="180x180" href="{{ url_for('static', filename='img/apple-touch-icon.png') }}" />
  <link rel="manifest" href="{{ url_for('static', filename='img/site.webmanifest') }}" />
</head>

<body>
  <header>
    <div class="container">
      <nav>
        <ul>
          <li><strong><a href="{{ url_for('general.index') }}">CastMail2List</a></strong></li>
        </ul>
        {%- if session._user_id %}
        <ul>
          <li><a href="{{ url_for('messages.index') }}">{{ _('Messages') }}</a></li>
          <li><a href="{{ url_for('lists.index') }}">{{ _('Lists') }}</a></li>
          <li><a href="{{ url_for('subscribers.index') }}">{{ _('Subscribers') }}</a></li>
          <li><a href="{{ url_for('logs.index') }}">{{ _('Logs') }}</a></li>
          <li><a href="{{ url_for('general.account') }}">{{ _('Account') }}</a></li>
          <li>
            <span id="theme-toggle" data-theme-toggle aria-label="Change to light theme" title="Change to light theme" class="icon sun"></span>
          </li>
        </ul>
        {%- endif %}
      </nav>
    </div>
  </header>
  <main class="container">
    {% block content %}{% endblock %}
  </main>
  <footer class="container">
    <hr />
    <p>
      Powered by <a href="https://github.com/mxmehl/castmail2list" target="_blank">CastMail2List</a>
      {%- if session._user_id %} {{ version_info }}{% endif %}
    </p>
  </footer>
  <script src="{{ url_for('static', filename='js/themeToggle.js') }}"></script>
</body>

</html>
