{% extends "base.html" %}

{% block title %}{{ _('Add New List') }}{% endblock %}

{% block content %}
<h2>{{ _('Add a New List') }}</h2>

{%- include "flash.html" %} {# Show flash messages #}

{%- from "macros.html" import list_form %}
{{ list_form(form, not retry) }}

{%- endblock %}
